# This file is part of the Falcon repository creator
# Copyright (C) 2005-2006 Dennis Kaarsemaker
# See the file named COPYING in the root of the source tree for license details

import os, statvfs, getpass, time
import falcon.util
import falcon.conf
import tempfile, shutil

isocache = []

diskdefines = """#define DISKNAME  Falcon %s iso - created by %s on %s
#define TYPE  binary
#define TYPEbinary  1
#define ARCH  %s
#define ARCH%s  1
#define DISKNUM  1
#define DISKNUM1  1
#define TOTALNUM  1
#define TOTALNUM1  1"""

def create_tree(topdir):
    if falcon.conf.pretend:
        return
    basedir = tempfile.mkdtemp(prefix='falcon-iso-', dir='/tmp')
    os.makedirs(os.path.join(basedir,'pool','falcon','iso'))
    # Make falcon update happy
    falcon.util.writefile(os.path.join(basedir,'pool','falcon','falcon.ini'),'[falcon]\n')
    # Make apt-cdrom happy
    falcon.util.writefile(os.path.join(basedir,'README.diskdefines'),diskdefines % (falcon.conf._f_version,
                           getpass.getuser(), time.strftime("%Y/%m/%d %H:%S"),
                           falcon.conf.architectures[0], falcon.conf.architectures[0]))
    os.mkdir(os.path.join(basedir,'.disk'))
    falcon.util.writefile(os.path.join(basedir,'.disk','base_installable'),'')
    falcon.util.writefile(os.path.join(basedir,'.disk','udeb_include'),'')
    falcon.util.writefile(os.path.join(basedir,'.disk','base_components'),'iso\n')
    falcon.util.writefile(os.path.join(basedir,'.disk','info'),
                          'Falcon %s iso - created by %s on %s' % (falcon.conf._f_version, getpass.getuser(), time.strftime("%Y/%m/%d %H:%S")))
    
    for path in isocache:
        (dir, file) = os.path.split(path)
        newdir = os.path.join(basedir, 'pool', 'falcon', 'iso', dir)
        if not os.path.exists(newdir):
            os.makedirs(newdir)
        os.symlink(os.path.join(os.getcwd(),path),os.path.join(newdir, file))
    return basedir

def check_size(topdir):
    # Required diskspace
    required = 0
    for (dir, dirs, files) in os.walk(topdir):
        for file in files:
            if file.endswith('.deb'):
                required += os.path.getsize(os.path.join(dir,file))
                f = os.path.join(dir, file).replace(topdir,'')
                if f[0] == '/':
                    f = f[1:]
                isocache.append(f)
    # Available diskspace in /tmp
    st = os.statvfs('/tmp')
    available = st[statvfs.F_BAVAIL] * st[statvfs.F_FRSIZE]
    
    return required / (1024*1024.0) + 3, available / (1024 * 1024.0)
    # 3 Extra megabytes for dists/  --^ (quite generous)

def create_iso(path):
    if falcon.conf.pretend:
        return
    # Remove all that's unneeded
    shutil.rmtree(os.path.join(path, 'morgue'))
    os.unlink(os.path.join(path, 'falcon.db'))
    os.unlink(os.path.join(path, 'pool','falcon','falcon.ini'))
    isofile = tempfile.mkstemp(prefix='falcon-iso-%s-' % getpass.getuser(), dir = '/tmp')
    os.close(isofile[0])
    isofile = isofile[1]
    args = '-allow-leading-dots -J -r -f -V "Falcon iso created on %s"' % time.strftime("%Y/%m/%d")

    if falcon.conf.verbose:
        print "mkisofs %s -o %s %s" % (args, isofile, path)
    falcon.util.run("mkisofs %s -o %s %s" % (args, isofile, path))
    return isofile

def cleanup(path):
    if falcon.conf.pretend:
        return
    if path.startswith('/tmp/falcon-iso-'):
        shutil.rmtree(path)
