# This file is part of the Falcon repository creator
# Copyright (C) 2005-2006 Dennis Kaarsemaker
# See the file named COPYING in the root of the source tree for license details

import os, sys
import time
import falcon.conf
import subprocess

def writefile(path, data):
    if not falcon.conf.pretend:
        olddata = readfile(path)
        if data == str(olddata):
            debug("Not overwriting unchanged file %s" % path)
        else:
            debug("Writing %d bytes to %s" % (len(str(data)), path))
            fd = open(path, 'w')
            fd.write(str(data))
            fd.close()

def readfile(path, ignore_errors = True):
    debug("Reading %s" % path)
    try:
        fd = open(path, 'r')
        d = fd.read()
        fd.close
        return d
    except:
        debug("File can't be read")
        if ignore_errors:
            return ''
        raise

def run(command, wd=None):
    if wd:
        debug("Executing in workdir %s: %s" % (wd, command))
    else:
        debug("Executing: %s" % command)
    process = subprocess.Popen(command, cwd = wd, executable = '/bin/bash', stdout = subprocess.PIPE, 
                               stderr = subprocess.PIPE, stdin = subprocess.PIPE, shell=True)
    stdout, stderr = process.communicate()
    ret = process.wait()
    if ret != 0:
        raise RuntimeError("%s failed with code %d\n\n%s" % (command, ret, stderr))
    return stdout

def intersect(a1,a2):
    if a1 == ['all']:
        return a2
    if a2 == ['all']:
        return a1
    return [x for x in a1 if x in a2]

output  = lambda x: sys.stdout.write('*  ' + x + "\n")
warning = lambda x: sys.stdout.write('W: ' + x + "\n")
error   = lambda x: sys.stderr.write('E: ' + x + "\n") or sys.exit(1)
def debug(x):
    if 'conf' in dir(falcon) and falcon.conf.verbose: print 'D: ' + x

outfunc = (output, warning, error, debug)

def yesno(question, default):
    while True:
        sys.stdout.write(question)
        if default: sys.stdout.write(" [Y] ")
        else:       sys.stdout.write(" [N] ")
        answer = raw_input()
        if answer in 'yY':
            return True
        if answer in 'nN':
            return False

def usage(exitcode):
    print """Usage: %s [-c configfile] [-p] [-v] [-h] action"
  -h, --help: Display this message
  -p, --pretend: Do not change/remove any files
  -v, --verbose: Verbose output
  -c configfile: Use configfile for configuration, default is /etc/falcon.ini

  action is either html, update, clean, or sync. update will re-scan the pool/
  directory, html will (re)create the HTML pages, clean will scan the pool/
  directory for old versions of packages and move these to the morgue/ dir. sync
  will rsync the archive to all known mirrors""" % sys.argv[0]
    sys.exit(exitcode)

def importusage(exitcode):
    print """Usage: %s -h -v [-c configfile] [-x] -r release -t \\
         component repository release component package
       %s -h -v [-c configfile] [-x] --cwd repository directory package
  
 It will download the source package files from the specified repository and
 place them inside the falcon-managed repository in your configuration or the
 current working directory""" % (sys.argv[0], sys.argv[0])
    sys.exit(exitcode)

def newer(file1, file2):
    try:
        return os.path.getmtime(file1) > os.path.getmtime(file2)
    except: # Exceptions occur when files don't exist
        return True

# The main decorator
def timer(func):
    def wrapper(*args, **kwargs):
        start = time.time()
        res = func(*args, **kwargs)
        stop = time.time()
        delta = stop - start
        if falcon.conf.timings:
            falcon.util.output("Function %s (args: %s, kwargs: %s) took %.3f seconds" %
                               (func.func_name, str(args), str(kwargs), delta))
        return res
    return wrapper

# Timer stack for non-decorator timers
timerstack = []
def timer_start(text=None):
    if falcon.conf.timings:
        timerstack.append((text,time.time()))

def timer_stop():
    if falcon.conf.timings:
        text, tm = timerstack.pop()
        falcon.util.output("%s took %.3f seconds" % (text, time.time() - tm))
